/****************************************************************************/
/*																			*/
/*	Module:			mb_fpp_io.c	(MicroBlaster FPP)							*/	
/*																			*/
/*					Copyright (C) Altera Corporation 2001					*/
/*																			*/
/*	Descriptions:	Defines all IO control functions. Operating system		*/
/*					is defined here. Functions are operating system 		*/
/*					dependent.												*/
/*																			*/
/*	Revisions:		1.0	2/28/03 Shee Kian Wong								*/
/*																			*/
/*																			*/
/****************************************************************************/

#include "mb_fpp_io.h"
#include <stdio.h>

#if PORT == WINDOWS_NT
#include <windows.h>
#else if PORT == EMBEDDED
/* include your library here */
#endif

/*////////////////////*/
/* Global Definitions */
/*////////////////////*/
#if PORT == WINDOWS_NT

#define	PGDC_IOCTL_GET_DEVICE_INFO_PP	0x00166A00L
#define PGDC_IOCTL_READ_PORT_PP			0x00166A04L
#define PGDC_IOCTL_WRITE_PORT_PP		0x0016AA08L
#define PGDC_HDLC_NTDRIVER_VERSION		2

#endif /*PORT == WINDOWS_NT*/

/*//////////////////*/
/* Global Variables */
/*//////////////////*/
#if PORT == WINDOWS_NT

HANDLE	nt_device_handle = INVALID_HANDLE_VALUE;

#else if PORT == EMBEDDED

/* define your global variables here */

#endif /* PORT == WINDOWS_NT */

#if PORT == WINDOWS_NT 
/* InitNtDriver and CloseNtDriver functions are for Windows NT only				*/

/********************************************************************************/
/*	Name:			InitNtDriver  												*/
/*																				*/
/*	Parameters:		None.          												*/
/*																				*/
/*	Return Value:	None.														*/
/*																				*/
/*	Descriptions:	Initiallize Windows NT Driver for ByteBlasterMV.			*/
/*																				*/
/********************************************************************************/
void InitNtDriver()
{
	int init_ok = 0;	/* Initialization OK */

	ULONG buffer[1];
	ULONG returned_length = 0;
	char nt_lpt_str[] = { '\\', '\\', '.', '\\', 'A', 'L', 'T', 'L', 'P', 'T', '1', '\0' }; 

	nt_device_handle = CreateFile( 
			nt_lpt_str,
			GENERIC_READ | GENERIC_WRITE,
			0,
			NULL,
			OPEN_EXISTING,
			FILE_ATTRIBUTE_NORMAL,
			NULL );

	if ( nt_device_handle == INVALID_HANDLE_VALUE )
		fprintf( stderr, "I/O Error: Cannot open device \"%s\"\n", nt_lpt_str );
	else
	{
		if ( DeviceIoControl(
				nt_device_handle,
				PGDC_IOCTL_GET_DEVICE_INFO_PP,
				(ULONG *) NULL,
				0,
				&buffer,
				sizeof(ULONG),
				&returned_length,
				NULL ))
		{
			if ( returned_length == sizeof( ULONG ) )
			{
				if (buffer[0] == PGDC_HDLC_NTDRIVER_VERSION)
				{
					init_ok = 1;
					fprintf( stdout, "Info: Port \"%s\" opened...\n", nt_lpt_str );
				}
				else
				{
					fprintf(stderr,
						"I/O Error:  device driver %s is not compatible\n(Driver version is %lu, expected version %lu.\n",
						nt_lpt_str,
						(unsigned long) buffer[0],
						(unsigned long) PGDC_HDLC_NTDRIVER_VERSION );
				}
			}	
			else
				fprintf(stderr, "I/O Error:  device driver %s is not compatible.\n", nt_lpt_str);		
		}

		if ( !init_ok )
		{
			fprintf( stderr, "I/O Error: DeviceIoControl not successful" );
			CloseHandle( nt_device_handle );
			nt_device_handle = INVALID_HANDLE_VALUE;
		}
	}

	if ( !init_ok )
	{
		fprintf( stderr, "Error: Driver initialization fail... Exiting...\n" );
		CloseNtDriver();
		exit(1);
	}
}

/********************************************************************************/
/*	Name:			CloseNtDriver 												*/
/*																				*/
/*	Parameters:		None.          												*/
/*																				*/
/*	Return Value:	None.														*/
/*																				*/
/*	Descriptions:	Close Windows NT Driver.									*/
/*																				*/
/********************************************************************************/
void CloseNtDriver()
{
	CloseHandle( nt_device_handle );
	nt_device_handle = INVALID_HANDLE_VALUE;
}

#else if PORT == EMBEDDED

/* put your code for driver initializing and closing codes here*/

#endif /* PORT == WINDOWS_NT */

/********************************************************************************/
/*	Name:			ReadPort													*/
/*																				*/
/*	Parameters:		int port       												*/
/*					- port number 0, 1, or 2. Index to parallel port base		*/
/*					  address.													*/
/*																				*/
/*	Return Value:	Integer, value of the port.									*/
/*																				*/
/*	Descriptions:	Read the value of the port registers.						*/
/*																				*/
/********************************************************************************/
int ReadPort( int port )
{
	int data = 0;

#if PORT == WINDOWS_NT
	int status = 0;
	int returned_length = 0;

	status = DeviceIoControl(
			nt_device_handle,			/* Handle to device					*/
			PGDC_IOCTL_READ_PORT_PP,	/* IO Control code for Read			*/
			(ULONG *)&port,				/* Buffer to driver.				*/
			sizeof(int),				/* Length of buffer in bytes.		*/
			(ULONG *)&data,				/* Buffer from driver.				*/
			sizeof(int),				/* Length of buffer in bytes.		*/
			(ULONG *)&returned_length,	/* Bytes placed in data_buffer.		*/
			NULL);						/* Wait for operation to complete	*/

	if ((!status) || (returned_length != sizeof(int)))
	{
		fprintf(stderr, "I/O error:  Cannot access device\n");
		CloseHandle(nt_device_handle);
		CloseNtDriver();
		exit(1);
	}

#else if PORT == EMBEDDED

/* put your code for reading from I/O port here									*/
/* please refer to "global variables" section of mblaster.c for important notes */

#endif /*PORT == WINDOWS_NT*/

	return (data & 0xff);
}

/********************************************************************************/
/*	Name:			FlushPort													*/
/*																				*/
/*	Parameters:		int port, int data											*/
/*					- port number 0, 1, or 2. Index to parallel port base		*/
/*					  address.													*/
/*					- value to written to port registers.						*/
/*																				*/
/*	Return Value:	None                       									*/
/*																				*/
/*	Descriptions:	Write [data] to [port] registers. When dump to Port0,		*/
/*					processes in [port_io_buffer] are dumped					*/
/*					when [PORT_IO_BUFFER_SIZE] is reached.						*/
/*																				*/
/********************************************************************************/
void FlushPort( int port, int data )
{
	int status = 0;

#if PORT == WINDOWS_NT
	int returned_length = 0;
	int buffer[2];

	buffer[0] = port;
	buffer[1] = data;

	status = DeviceIoControl(
			nt_device_handle,			/* Handle to device.				*/
			PGDC_IOCTL_WRITE_PORT_PP,	/* IO Control code for write.		*/
			(ULONG *)&buffer,			/* Buffer to driver.				*/
			2 * sizeof(int),			/* Length of buffer in bytes.		*/
			(ULONG *)NULL,				/* Buffer from driver.  Not used.	*/
			0,							/* Length of buffer in bytes.		*/
			(ULONG *)&returned_length,	/* Bytes returned.  Should be zero. */
			NULL);						/* Wait for operation to complete.	*/

	if ((!status) || (returned_length != 0))
	{
		fprintf(stderr, "I/O Error:  Cannot access device!\n");
		CloseHandle(nt_device_handle);
		exit(1);
	}

#else if PORT == EMBEDDED

/* put your code for flushing the data in the I/O port here	     				*/
/* please refer to "global variables" section of mblaster.c for important notes */

#endif /*PORT == WINDOWS_NT*/

}

/********************************************************************************/
/*	Name:			Dump2PortIO													*/
/*																				*/
/*	Parameters:		int port, int data			 								*/
/*					- port of the signal 										*/
/*					- value to be dumped to the port.							*/
/*																				*/
/*	Return Value:	None.														*/
/*																				*/
/*	Descriptions:	Dump [data] to [port]										*/
/*																				*/
/********************************************************************************/
void Dump2PortIO( int port, int data )
{
/*------------------------------------------------------------------------------*/
/*	Notes																		*/
/*------------------------------------------------------------------------------*/
/*																				*/
/* DATA[7:0] requires continuous clocking, i.e. 1 clock cycle sends 1 byte of	*/
/* data. Due to the parallel port architecture limitations, DATA and DCLK       */
/* cannot be  driven out at the same time. Dump2PortIO first drive DCLK=1 to    */
/* device, followed by 1 byte configuration data, and finally DCLK=0.           */
/*																				*/
/* Although DCLK is driven in H->L sequence, DATA[7:0] is still latched on      */
/* rising edge of DCLK.															*/
/*																				*/

	int dclk0 = 0x1;	/*DCLK = LOW  */
	int dclk1 = 0x0;	/*DCLK = HIGH */
	
	if (port == 2) 
	{
		FlushPort( port, data );
	}
	else	
	{
		FlushPort( 2, dclk1 );  
		FlushPort( port, data );
		FlushPort( 2, dclk0 );
	}
}

int fopen_rbf( char argv[], char* mode)
{
	FILE* file_id;

	file_id = fopen( argv, mode );

	return (int) file_id;
}

int	fclose_rbf( int file_id)
{
	fclose( (FILE*) file_id);

	return 0;
}

int fseek_rbf( int finputid, int start, int end )
{
	int seek_position;

	seek_position = fseek( (FILE*) finputid, start, end );

	return seek_position;
}

int ftell_rbf( int finputid )
{
	int file_size;

	file_size = ftell( (FILE*) finputid );

	return file_size;
}

int fgetc_rbf( int finputid )
{
	int one_byte;

	one_byte = fgetc( (FILE*) finputid );

	return one_byte;
}