/****************************************************************************/
/*																			*/
/*	Module:			mblaster_fpp.c	(MicroBlaster FPP)						*/	
/*																			*/
/*					Copyright (C) Altera Corporation 2003					*/
/*																			*/
/*	Descriptions:	Main source file that manages configuration process.	*/
/*					IO related control functions are defined in mb_fpp_io.c	*/
/*					module.													*/
/*																			*/
/*	Revisions:		1.0	2/28/03 Shee Kian Wong       						*/
/*																			*/
/*																			*/
/****************************************************************************/
#include "mb_fpp_io.h"
#include <stdio.h>

#if PORT == WINDOWS_NT
#include <conio.h>
#endif

/*////////////////////*/
/* Global Definitions */
/*////////////////////*/
#define NSTATUS			0
#define CONF_DONE		1

#define S_SET			0			/* SEEK_SET */
#define S_CUR			1			/* SEEK_CUR */
#define S_END			2			/* SEEK_END */

/*//////////////////*/
/* User Definitions */
/*//////////////////*/
#define INIT_CYCLE			50		/* #clock cycle after configuration is done and before USER mode */
#define RECONF_COUNT_MAX	5		/* #auto-reconfiguration when error found						 */
#define	CHECK_EVERY_X_BYTE  10240	/* Check NSTATUS pin for error for every X bytes. DO NOT put '0' */
#define CLOCK_X_CYCLE		0		/* Clock another 'x' cycles during initialization (optional)	 */

/*///////////////////////*/
/* Functions Prototyping */
/*///////////////////////*/
int		CheckSignal			( int signal );
void    Dump2Port			( int port, int data );
void	PrintError			( int configuration_count );
void	ProcessFileInput	( int finputid );
void	ResetPort			( void );

/*//////////////////*/
/* Global variables */
/*//////////////////*/

/*----------------------------------------------------------------------------------------------------------*/
/*	Important Notes																							*/
/*----------------------------------------------------------------------------------------------------------*/
/*																											*/
/* The following variables are used throughout this file and specifically applies to PORT == WINDOWS_NT.	*/
/* To port to other platforms, e.g. EMBEDDED, user should modify Dump2Port, ReadPort and FlushPort			*/
/* functions to translate the signals to I/O port architecture of your system.								*/
/* The summary of Port and Bit Position of parallel port architecture is shown below:						*/
/*																											*/
/*				Bit       7        6        5        4        3        2        1        0					*/
/*	(DATA)		Port 0  DATA7    DATA6    DATA5    DATA4    DATA3    DATA2    DATA1    DATA0				*/
/*	(STATUS)	Port 1 !CONF_DONE  -        -      NSTATUS    -        -        -        -					*/ 
/*	(CONTROL)	Port 2    -        -        -        -    !NCONFIG     -        -      !DCLK /* !-inverted	*/
/*																											*/
/* Port 0 (DATA) and Port 2 (CONTROL) are output ports whereas Port 1 (STATUS) is an input port			    */
																																							
/* sig_port_maskbit[X] - Let us know the mask bit position of NSTATUS & CONF_DONE signals					*/
/*  					 X=0: NSTATUS; X=1: CONF_DONE														*/

int		sig_port_maskbit[2]	= { 0x10, 0x80 };
								
/* port_reset[Z]	   - Holds the reset value of the output ports (Port 0 and Port 2)						*/	 
/*						 Z=0: Port 0; Z=1: Port 2															*/

int		port_reset[2]		= { 0xFF, 0x0 };	

/********************************************************************************/
/*	Name:			Main          												*/
/*																				*/
/*	Parameters:		int argc, char* argv[]										*/
/*					- number of argument.										*/
/*					- argument character pointer.								*/ 
/*																				*/
/*	Return Value:	None.														*/
/*																				*/
/*	Descriptions:	Open programming file and initialize driver if required		*/
/*					(WINDOWS NT).												*/
/*																				*/
/********************************************************************************/
void main( int argc, char* argv[] )
{
	int file_id;

	/* Introduction */
	fprintf( stdout, "\n====================================\n" );
	fprintf( stdout, "MicroBlaster (MBlaster) Version 1.0" );
	fprintf( stdout, "\nAltera Corporation " );
	fprintf( stdout, "\n====================================\n" );

	/* Syntax check */
	if ( argc != 2 )
	{
		fprintf( stderr, "Error: Invalid number of argument! \nSyntax: \"%s <programming_file_name>\"\n", argv[0] );
		return;
	}

	/* Open programming file as READ and in BINARY */
	file_id = fopen_rbf( argv[1], "rb" );

	if ( file_id )
		fprintf( stdout, "Info: Programming file: \"%s\" opened...\n", argv[1] );
	else
	{
		fprintf( stderr, "Error: Could not open programming file: \"%s\"\n", argv[1] );
		return;
	}

	/* Initialize driver */
#if PORT == WINDOWS_NT
	InitNtDriver();
#endif /* PORT == WINDOWS_NT */

	/* Processing and Programming */
	ProcessFileInput( file_id );

	/* Close driver and programming file */
#if PORT == WINDOWS_NT
	CloseNtDriver();
#endif /* PORT == WINDOWS_NT */

	if ( file_id )
		fclose_rbf(file_id);
}

/********************************************************************************/
/*	Name:			ProcessFileInput											*/
/*																				*/
/*	Parameters:		FILE* finputid												*/
/*					- programming file pointer.									*/
/*																				*/
/*	Return Value:	None.														*/
/*																				*/
/*	Descriptions:	Get programming file size, parse through every single byte	*/
/*					and dump to parallel port.									*/
/*																				*/
/*					For every [CHECK_EVERY_X_BYTE] bytes, NSTATUS pin is		*/
/*					checked for error. When the file size is reached, CONF_DONE	*/
/*					pin is checked for configuration status. Then, another		*/
/*					[INIT_CYCLE] clock cycles are dumped to initialize			*/
/*					the device.													*/
/*																				*/
/*					Configuration process is restarted whenever error found.	*/
/*					The maximum number of auto-reconfiguration is set in		*/
/*					[RECONF_COUNT_MAX].											*/
/*																				*/
/********************************************************************************/
void ProcessFileInput( int finputid )
{
	int			program_done		= 0;					/* configuration & initialization process done = 1		   	 */
	int			seek_position		= 0;					/* file pointer position									 */
	long int	file_size			= 0;					/* programming file size									 */
	int			configuration_count = RECONF_COUNT_MAX;		/* # reprogramming after error								 */
	int			one_byte			= 0;					/* the byte to program										 */
	long int	i					= 0;					/* standard counter variable								 */
	int			CONF_DONE_ok		= 1;					/* CONF_DONE pin. '1' - error								 */
	int			nstatus_ok			= 0;					/* NSTATUS pin. '0' - error									 */
	int			clock_x_cycle		= CLOCK_X_CYCLE;		/* Clock another 'x' cycles during initialization (optional) */

	/* Get file size */
	seek_position = fseek_rbf( finputid, 0, S_END );

	if ( seek_position )
	{
		fprintf( stderr, "Error: End of file could not be located!" );
		return;
	}

	file_size	= ftell_rbf( finputid );
	fprintf( stdout, "Info: Programming file size: %ld\n", file_size );
	
	/* Start configuration */
	while ( !program_done && (configuration_count>0) )
	{
		/* Reset file pointer and parallel port registers */
		fseek_rbf( finputid, 0, S_SET );
		ResetPort( );
		
		fprintf( stdout, "\n***** Start configuration process *****\nPlease wait...\n" );

		/* Drive a transition of 0 to 1 to NCONFIG to indicate start of configuration */
		Dump2Port( 2, 0x8 ); /* NCONFIG = LOW  */
		Dump2Port( 2, 0x0 ); /* NCONFIG = HIGH */

		/* Loop through every single byte */
		for ( i = 0; i < file_size; i++ )
		{
			/*one_byte = fgetc( (FILE*) finputid );*/
			one_byte = fgetc_rbf( finputid );

			/* Program a byte */
			Dump2Port( 0, one_byte );

			/* Check for error through NSTATUS for every 10KB programmed and the last byte */
			if ( !(i % CHECK_EVERY_X_BYTE) || (i == file_size - 1) )
			{
				nstatus_ok = CheckSignal( NSTATUS );

				if ( !nstatus_ok )
				{
					PrintError( configuration_count-1 );
					
					program_done = 0;
					break;
				}
				else 
					program_done = 1;
			}
		}

		configuration_count--;

		if ( !program_done )
			continue;

		/* Configuration end */
		/* Check CONF_DONE that indicates end of configuration */
		CONF_DONE_ok = CheckSignal( CONF_DONE );

		if ( CONF_DONE_ok )
		{
			fprintf( stderr, "Error: Configuration done but contains error... CONF_DONE is %s\n", (CONF_DONE_ok? "LOW":"HIGH") );
			program_done = 0;
			PrintError( configuration_count );			
			if ( configuration_count == 0 )
				break;
		}
	
		/* if contain error during configuration, restart configuration */
		if ( !program_done )
			continue;

		/* program_done = 1; */

		/* Start initialization */
		/* Clock another (INIT_CYCLE) cycles to initialize the device */
		for ( i = 0; i < INIT_CYCLE; i++ )
		{
			Dump2Port( 2, 0x0 );
			Dump2Port( 2, 0x1 );
		}
		/* Initialization end */

		nstatus_ok = CheckSignal( NSTATUS );
		CONF_DONE_ok = CheckSignal( CONF_DONE );

		if ( !nstatus_ok || CONF_DONE_ok )
		{
			fprintf( stderr, "Error: Initialization finish but contains error: NSTATUS is %s and CONF_DONE is %s. Exiting...", (nstatus_ok?"HIGH":"LOW"), (CONF_DONE_ok?"LOW":"HIGH") );
			program_done = 0; 
			configuration_count = 0; /* No reconfiguration */
		}
	}

	/* add another 'x' clock cycles to make sure the device is initialized (certain cases) */
	if ( clock_x_cycle > 0 )
	{
		fprintf( stdout, "Info: Clock another %d cycles in device initialization process...\n", clock_x_cycle );
		for ( i = 0; i < CLOCK_X_CYCLE; i++ )
		{
			Dump2Port( 2, 0x0 );
			Dump2Port( 2, 0x1 );
		}
	}

	if ( !program_done )
	{
		fprintf( stderr, "\nError: Configuration not successful! Error encountered...\n" );
		ResetPort( );
		return;
	}

	fprintf( stdout, "\nInfo: Configuration successful!\n" );

	ResetPort( );
}

/********************************************************************************/
/*	Name:			CheckSignal													*/
/*																				*/
/*	Parameters:		int signal						 							*/
/*					- name of the signal										*/
/*																				*/
/*	Return Value:	Integer, the value of the signal. '0' is returned if the	*/
/*					value of the signal is LOW; if not, the signal is HIGH.		*/
/*																				*/
/*	Descriptions:	Return the value of the signal.								*/
/*																				*/
/********************************************************************************/

int CheckSignal( int signal )
{
	int value	= 1;
	int	readout = 0;

	readout = ReadPort( 1 );
	value = readout & sig_port_maskbit[signal];

	return value;
}

/********************************************************************************/
/*	Name:			ResetPort													*/
/*																				*/
/*	Parameters:		none						 								*/
/*																				*/
/*	Return Value:	None.														*/
/*																				*/
/*	Descriptions:	Set the parallel port registers to particular values.		*/
/*																				*/
/********************************************************************************/

void ResetPort( )
{
	FlushPort( 0, port_reset[0] );
	FlushPort( 2, port_reset[1] );
}

/********************************************************************************/
/*	Name:			Dump2Port													*/
/*																				*/
/*	Parameters:		int port, int data			 								*/
/*					- port of the signal 										*/
/*					- value to be dumped to the port.							*/
/*																				*/
/*	Return Value:	None.														*/
/*																				*/
/*	Descriptions:	Dump [data] to [port]										*/
/*																				*/
/********************************************************************************/

void Dump2Port( int port, int data )
{
	Dump2PortIO( port, data );
}

/********************************************************************************/
/*	Name:			PrintError 													*/
/*																				*/
/*	Parameters:		int configuration_count										*/
/*					- # auto-reconfiguration left  								*/
/*																				*/
/*	Return Value:	None.														*/
/*																				*/
/*	Descriptions:	Print error message to standard error.						*/
/*																				*/
/********************************************************************************/

void PrintError( int configuration_count )
{
	if ( configuration_count == 0 )
		fprintf( stderr, "Error: Error in configuration #%d... \nError: Maximum number of reconfiguration reached. Exiting...\n", (RECONF_COUNT_MAX-configuration_count) );
	else
	{
		fprintf( stderr, "Error: Error in configuration #%d... Restart configuration. Ready? <Press any key to continue>\n", (RECONF_COUNT_MAX-configuration_count) );
#if PORT == WINDOWS_NT
		getch();
#endif
	}
}
